"""
Internal data handler module. Provides dataclass-based structures representation.
"""

from . import animation, flags, mesh, scene, skeleton, texture, vectors


__all__ = (
    "animation",
    "flags",
    "mesh",
    "scene",
    "skeleton",
    "texture",
    "vectors",
)